@@ -22,13 +22,14 @@ import ctypes.util
 import platform
 import os.path
 import sys
-import inspect
 
 class Enum(object):
-    def __init__(self, member_dict):
+    def __init__(self, member_dict, scope_dict=None):
+        if scope_dict is None:
+            # Affect caller's locals, not this module's.
+            scope_dict = sys._getframe(1).f_locals
         forward_dict = {}
         reverse_dict = {}
-        global_dict = inspect.stack()[1][0].f_globals
         next_value = 0
         for name, value in member_dict.items():
             if value is None:
@@ -39,7 +40,7 @@ class Enum(object):
                 raise ValueError('Multiple names for value %r: %r, %r' %
                     (value, reverse_dict[value], name))
             reverse_dict[value] = name
-            global_dict[name] = value
+            scope_dict[name] = value
         self.forward_dict = forward_dict
         self.reverse_dict = reverse_dict
 
